//library international_phone_input;

import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:international_phone_number/src/phone_service.dart';

import 'country.dart';

class InternationalPhoneInput extends StatefulWidget {
  final void Function(String phoneNumber, String internationalizedPhoneNumber,
      String isoCode, String dialCode) onPhoneNumberChange;
  final String initialPhoneNumber;
  final String initialSelection;
  final String errorText;
  final String hintText;
  final String labelText;
  final TextStyle errorStyle;
  final TextStyle hintStyle;
  final TextStyle labelStyle;
  final int errorMaxLines;
  final List<String> enabledCountries;
  final InputDecoration decoration;
  final bool showCountryCodes;
  final bool showCountryFlags;
  final Widget dropdownIcon;
  final InputBorder border;

  const InternationalPhoneInput(
      {
      required this.onPhoneNumberChange,
      required this.initialPhoneNumber,
      required this.initialSelection,
      required this.errorText,
      required this.hintText,
      required this.labelText,
      required this.errorStyle,
      required this.hintStyle,
      required this.labelStyle,
      this.enabledCountries = const [],
      required this.errorMaxLines,
      required this.decoration,
      this.showCountryCodes = true,
      this.showCountryFlags = true,
      required this.dropdownIcon,
      required this.border});

  static Future<String> internationalizeNumber(String number, String iso) {
    return PhoneService.getNormalizedPhoneNumber(number, iso);
  }

  @override
  _InternationalPhoneInputState createState() =>
      _InternationalPhoneInputState();
}

class _InternationalPhoneInputState extends State<InternationalPhoneInput> {
  Country selectedItem = Country(name: "", code: "", flagUri: "", dialCode: "");
  List<Country> itemList = [];

  String ?errorText;
  String ?hintText;
  String ?labelText;

  TextStyle ?errorStyle;
  TextStyle ?hintStyle;
  TextStyle ?labelStyle;

  int ?errorMaxLines;

  bool hasError = false;
  bool ?showCountryCodes;
  bool ?showCountryFlags;

  InputDecoration ?decoration;
  Widget ?dropdownIcon;
  InputBorder ?border;

  _InternationalPhoneInputState();

  final phoneTextController = TextEditingController();

  @override
  void initState() {
    errorText = widget.errorText;
    hintText = widget.hintText ;
    labelText = widget.labelText;
    errorStyle = widget.errorStyle;
    hintStyle = widget.hintStyle;
    labelStyle = widget.labelStyle;
    errorMaxLines = widget.errorMaxLines;
    decoration = widget.decoration;
    showCountryCodes = widget.showCountryCodes;
    showCountryFlags = widget.showCountryFlags;
    dropdownIcon = widget.dropdownIcon;

    phoneTextController.addListener(_validatePhoneNumber);
    phoneTextController.text = widget.initialPhoneNumber;

    _fetchCountryData().then((list) {
      Country preSelectedItem;

      if (widget.initialSelection != null) {
        preSelectedItem = list.firstWhere(
            (e) =>
                (e.code.toUpperCase() ==
                    widget.initialSelection.toUpperCase()) ||
                (e.dialCode == widget.initialSelection.toString()),
            orElse: () => list[0]);
      } else {
        preSelectedItem = list[0];
      }

      setState(() {
        itemList = list;
        selectedItem = preSelectedItem;
      });
    });

    super.initState();
  }

  _validatePhoneNumber() {
    String phoneText = phoneTextController.text;
    if (phoneText.isNotEmpty) {
      PhoneService.parsePhoneNumber(phoneText, selectedItem.code)
          .then((isValid) {
        setState(() {
          hasError = !isValid;
        });

        if (isValid) {
          PhoneService.getNormalizedPhoneNumber(phoneText, selectedItem.code)
              .then((number) {
            widget.onPhoneNumberChange(phoneText, number, selectedItem.code, selectedItem.dialCode);
          });
        } else {
          widget.onPhoneNumberChange('', '', selectedItem.code, selectedItem.dialCode);
        }
      });
    }
  }

  Future<List<Country>> _fetchCountryData() async {
    var list = await DefaultAssetBundle.of(context)
        .loadString('packages/international_phone_number/assets/countries.json');
    List<dynamic> jsonList = json.decode(list);

    List<Country> countries = List<Country>.generate(jsonList.length, (index) {
      Map<String, String> elem = Map<String, String>.from(jsonList[index]);
      if (widget.enabledCountries.isEmpty) {
        return Country(
            name: elem['en_short_name']!,
            code: elem['alpha_2_code']!,
            dialCode: elem['dial_code']!,
            flagUri: 'assets/flags/${elem['alpha_2_code']!.toLowerCase()}.png');
      } else if (widget.enabledCountries.contains(elem['alpha_2_code']) ||
          widget.enabledCountries.contains(elem['dial_code'])) {
        return Country(
            name: elem['en_short_name']!,
            code: elem['alpha_2_code']!,
            dialCode: elem['dial_code']!,
            flagUri: 'assets/flags/${elem['alpha_2_code']!.toLowerCase()}.png');
      }else{
        return Country(name: "", code: "", flagUri: "", dialCode: "");
      }
    });

    countries.removeWhere((value) => value == null);

    return countries;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        children: <Widget>[
          DropdownButtonHideUnderline(
            child: Padding(
              padding: const EdgeInsets.only(top: 8),
              child: DropdownButton<Country>(
                value: selectedItem,
                icon: Padding(
                  padding:
                      EdgeInsets.only(bottom: (decoration != null) ? 6 : 0),
                  child: dropdownIcon ?? const Icon(Icons.arrow_drop_down),
                ),
                onChanged: (Country ?newValue) {
                  setState(() {
                    selectedItem = newValue!;
                  });
                  _validatePhoneNumber();
                },
                items: itemList.map<DropdownMenuItem<Country>>((Country value) {
                  return DropdownMenuItem<Country>(
                    value: value,
                    child: Container(
                      padding: const EdgeInsets.only(bottom: 5.0),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                        children: <Widget>[
                          if (showCountryFlags!) ...[
                            Image.asset(
                              value.flagUri,
                              width: 32.0,
                              package: 'international_phone_number',
                            )
                          ],
                          if (showCountryCodes!) ...[
                            const SizedBox(width: 4),
                            Text(value.dialCode)
                          ]
                        ],
                      ),
                    ),
                  );
                }).toList(),
              ),
            ),
          ),
          Flexible(
              child: TextField(
            keyboardType: TextInputType.phone,
            controller: phoneTextController,
            decoration: decoration ??
                InputDecoration(
                  hintText: hintText,
                  labelText: labelText,
                  errorText: hasError ? errorText : null,
                  hintStyle: hintStyle,
                  errorStyle: errorStyle,
                  labelStyle: labelStyle,
                  errorMaxLines: errorMaxLines ?? 3,
                  border: border,
                ),
          ))
        ],
      ),
    );
  }
}
